@@ -44,6 +44,7 @@ except ImportError:
     pdict = None
 
 SLEEP_TIME = 0.1 # how long to sleep when waiting for network activity
+DEFAULT_PRIORITY = 1 # default queue priority
 
 
 
@@ -115,12 +116,14 @@ class Download:
         what to return when no content can be downloaded
     pattern:
         a regular expression that the downloaded HTML has to match to be considered a valid download
+    acceptable_errors:
+        a list contains all acceptable HTTP codes, don't try downloading for them e.g. no need to retry for 404 error
     """
 
     def __init__(self, cache=None, cache_file=None, read_cache=True, write_cache=True, use_network=True, 
             user_agent=None, timeout=30, delay=5, proxies=None, proxy_file=None, max_proxy_errors=5,
-            opener=None, headers=None, data=None, num_retries=0, num_redirects=1,
-            force_html=False, force_ascii=False, max_size=None, default='', pattern=None):
+            opener=None, headers=None, data=None, num_retries=0, num_redirects=0,
+            force_html=False, force_ascii=False, max_size=None, default='', pattern=None, acceptable_errors=None, **kwargs):
         socket.setdefaulttimeout(timeout)
         need_cache = read_cache or write_cache
         if pdict and need_cache:
@@ -129,7 +132,7 @@ class Download:
         else:
             self.cache = None
             if need_cache:
-                common.logger.info('Cache disabled because could not import pdict')
+                common.logger.warning('Cache disabled because could not import pdict')
 
         self.settings = adt.Bag(
             read_cache = read_cache,
@@ -149,9 +152,12 @@ class Download:
             force_ascii = force_ascii,
             max_size = max_size,
             default = default,
-            pattern = pattern
+            pattern = pattern,
+            acceptable_errors = acceptable_errors
         )
         self.last_load_time = self.last_mtime = time.time()
+        self.num_downloads = self.num_errors = 0
+
 
     proxy_performance = ProxyPerformance()
     def get(self, url, **kwargs):
@@ -168,6 +174,7 @@ class Download:
         self.final_url = None # for tracking redirects
         self.response_code = '' # keep response code
         self.response_headers = {} # keep response headers
+        self.downloading_error = None # keep downloading error
         self.num_downloads = self.num_errors = 0 # track the number of downloads made
                 
         # update settings with any local overrides
@@ -221,7 +228,7 @@ class Download:
                     for proxy in failed_proxies:
                         if Download.proxy_performance.error(self.proxy) > settings.max_proxy_errors:
                             # this proxy has had too many errors so remove
-                            common.logger.info('Removing unstable proxy from list after %d consecutive errors: %s' % (settings.max_proxy_errors, self.proxy))
+                            common.logger.warning('Removing unstable proxy from list after %d consecutive errors: %s' % (settings.max_proxy_errors, self.proxy))
                             settings.proxies.remove(self.proxy)
             else:
                 # download failed - try again
@@ -235,7 +242,7 @@ class Download:
                 redirect_url = self.get_redirect(url=url, html=html)
                 if redirect_url:
                     # found a redirection
-                    common.logger.info('%s redirecting to %s' % (url, redirect_url))
+                    common.logger.debug('%s redirecting to %s' % (url, redirect_url))
                     settings.num_redirects -= 1
                     html = self.get(redirect_url, **settings) or ''
                     # make relative links absolute so will still work after redirect
@@ -254,6 +261,32 @@ class Download:
         return html or settings.default 
 
 
+    def exists(self, url):
+        """Do a HEAD request to check whether webpage exists
+        """
+        success = False
+        key = self.get_key(url, 'head')
+        try:
+            if self.cache and self.settings.read_cache:
+                success = self.cache[key]
+            else:
+                raise KeyError('No cache')
+        except KeyError:
+            # have not downloaded yet
+            request = urllib2.Request(url)
+            request.get_method = lambda : 'HEAD'
+            try:
+                response = urllib2.urlopen(request)
+            except Exception, e:
+                common.logger.warning('HEAD check miss: %s %s' % (url, e))
+            else:
+                success = True
+                common.logger.info('HEAD check hit: %s' % url)
+            if self.cache:
+                self.cache[key] = success
+        return success
+
+
     def get_key(self, url, data=None):
         """Create key for caching this request
         """
@@ -276,10 +309,10 @@ class Download:
             content must be ASCII
         """
         if max_size is not None and len(html) > max_size:
-            common.logger.info('Too big: %s' % len(html))
+            common.logger.info('Webpage is too big: %s' % len(html))
             html = '' # too big to store
         elif force_html and not common.is_html(html):
-            common.logger.info('Not html')
+            common.logger.info('Webpage is not html')
             html = '' # non-html content
         elif force_ascii:
             html = common.to_ascii(html) # remove non-ascii characters
@@ -341,17 +374,24 @@ class Download:
                 if pattern and not re.compile(pattern, re.DOTALL | re.IGNORECASE).search(content):
                     # invalid result from download
                     content = None
-                    common.logger.info('Content did not match expected pattern - %s' % url)
+                    common.logger.warning('Content did not match expected pattern - %s' % url)
                 self.response_code = '200'
                 self.response_headers = dict(response.headers)
         except Exception, e:
+            self.downloading_error = str(e)
             if hasattr(e, 'code'):
-                self.response_code = e.code
+                self.response_code = str(e.code)
             if hasattr(e, 'read'):
-                self.error_content = e.read()
+                try:
+                    self.error_content = e.read()
+                except Exception, e:
+                    self.error_content = ''
             # so many kinds of errors are possible here so just catch them all
-            common.logger.info('Download error: %s %s' % (url, e))
-            content, self.final_url = None, url
+            common.logger.warning('Download error: %s %s' % (url, e))
+            if not self.settings.acceptable_errors or self.response_code not in self.settings.acceptable_errors:
+                content, self.final_url = None, url
+            else:
+                content, self.final_url = self.settings.default, url
         return content
 
 
@@ -392,7 +432,7 @@ class Download:
                     common.logger.debug('Reloaded proxies from updated file.')
 
 
-    def geocode(self, address, delay=5, read_cache=True):
+    def geocode(self, address, delay=5, read_cache=True, num_retries=1):
         """Geocode address using Google's API and return dictionary of useful fields
 
         address:
@@ -403,7 +443,7 @@ class Download:
             whether to load content from cache when exists
         """
         url = 'http://maps.google.com/maps/api/geocode/json?address=%s&sensor=false' % urllib.quote_plus(address)
-        html = self.get(url, delay=delay, read_cache=read_cache)
+        html = self.get(url, delay=delay, read_cache=read_cache, num_retries=num_retries)
         results = collections.defaultdict(str)
         if html:
             try:
@@ -413,7 +453,7 @@ class Download:
                 return {}
             for result in geo_data.get('results', []):
                 for e in result['address_components']:
-                    types, value = e['types'], e['long_name']
+                    types, value, abbrev = e['types'], e['long_name'], e['short_name']
                     if 'street_number' in types:
                         results['number'] = value
                     elif 'route' in types:
@@ -424,12 +464,14 @@ class Download:
                         results['suburb'] = value
                     elif 'administrative_area_level_1' in types:
                         results['state'] = value
+                        results['state_code'] = abbrev
                     elif 'administrative_area_level_2' in types:
                         results['county'] = value
                     elif 'administrative_area_level_3' in types:
                         results['district'] = value
                     elif 'country' in types:
                         results['country'] = value
+                        results['country_code'] = value
                 results['full_address'] = result['formatted_address']
                 m = re.compile(r'"location" : {\s*"lat" : ([\d\-\.]+),\s*"lng" : ([\d\-\.]+)').search(html)
                 if m:
@@ -442,9 +484,9 @@ class Download:
                 results['address'] = (results['number'] + ' ' + results['street']).strip()
         if not results:
             # error geocoding - try again later
-            common.logger.debug('delete invalid geocode')
+            common.logger.debug('Delete invalid geocode')
             if self.cache:
-                del self.cache[url]
+                self.cache[url] = ''
         return results
 
 
@@ -616,7 +658,7 @@ class Download:
 
 
 
-def threaded_get(url=None, urls=None, num_threads=10, dl=None, cb=None, depth=None, wait_finish=True, reuse_queue=False, max_queue=100, **kwargs):
+def threaded_get(url=None, urls=None, num_threads=10, dl=None, cb=None, depth=None, wait_finish=True, reuse_queue=False, max_queue=1000, **kwargs):
     """Download these urls in parallel
 
     url:
@@ -651,7 +693,7 @@ def threaded_get(url=None, urls=None, num_threads=10, dl=None, cb=None, depth=No
         """Thread for downloading webpages
         """
         processing = collections.deque() # to track whether are still downloading
-        downloading = set() # the URL's that are currently being downloaded
+        discovered = {} # the URL's that have been discovered
 
         def __init__(self):
             threading.Thread.__init__(self)
@@ -661,7 +703,6 @@ def threaded_get(url=None, urls=None, num_threads=10, dl=None, cb=None, depth=No
             queue = pdict.Queue(settings.queue_file)
 
             while seed_urls or DownloadThread.processing:
-                queue_size = None
                 # keep track that are processing url
                 DownloadThread.processing.append(1) 
                 try:
@@ -674,7 +715,6 @@ def threaded_get(url=None, urls=None, num_threads=10, dl=None, cb=None, depth=No
                     time.sleep(SLEEP_TIME)
 
                 else:
-                    DownloadThread.downloading.add(url)
                     try:
                         # download this url
                         html = dl(D, url, **kwargs) if dl else D.get(url, **kwargs)
@@ -685,41 +725,42 @@ def threaded_get(url=None, urls=None, num_threads=10, dl=None, cb=None, depth=No
 
                             except Exception, e:
                                 # catch any callback error to avoid losing thread
-                                common.logger.error('in callback for: ' + str(url) + '\n' + traceback.format_exc())
+                                common.logger.exception('\nIn callback for: ' + str(url))
 
                             else:
-                                if cb_urls:
-                                    # add these URL's to crawl queue
+                                # add these URL's to crawl queue
+                                for cb_url in cb_urls or []:
                                     if isinstance(cb_urls, dict):
-                                        keys, priorities = cb_urls.keys(), cb_urls
+                                        DownloadThread.discovered[cb_url] = cb_urls[cb_url]
                                     else:
-                                        keys, priorities = cb_urls, {}
-                                    lock.acquire()
-                                    queue.push(keys, priorities)
-                                    lock.release()
-                                    queue_size = len(queue)
-                                if not seed_urls:
-                                    lock.acquire()
-                                    if not seed_urls:
-                                        # get next batch of URLs from cache
-                                        for queued_url in queue.pull(limit=max_queue):
-                                            if queued_url not in DownloadThread.downloading:
-                                                # is not currently processing
-                                                seed_urls.append(queued_url)
-                                    lock.release()
+                                        DownloadThread.discovered[cb_url] = DEFAULT_PRIORITY
+                                            
+                                if len(seed_urls) < max_queue:
+                                    # need to request more queue
+                                    if DownloadThread.discovered or len(queue) > 0:
+                                        # there are outstanding in the queue
+                                        if lock.acquire(False):
+                                            # no other thread is downloading
+                                            common.logger.debug('Loading from queue: %d' % len(seed_urls))
+                                            discovered = []
+                                            while DownloadThread.discovered:
+                                                discovered.append(DownloadThread.discovered.popitem())
+                                            queue.push(discovered)
+                                            # get next batch of URLs from cache
+                                            seed_urls.extend(queue.pull(limit=max_queue))
+                                            lock.release()
                     finally:
                         # have finished processing
                         # make sure this is called even on exception to avoid eternal loop
                         DownloadThread.processing.pop()
-                        DownloadThread.downloading.discard(url)
                     # update the crawler state
                     # no download or error so must have read from cache
                     num_caches = 0 if D.num_downloads or D.num_errors else 1
-                    state.update(num_downloads=D.num_downloads, num_errors=D.num_errors, num_caches=num_caches, queue_size=queue_size)
+                    state.update(num_downloads=D.num_downloads, num_errors=D.num_errors, num_caches=num_caches, queue_size=len(queue))
+
 
-    
     queue = pdict.Queue(settings.queue_file)
-    if reuse_queue or '--use-queue' in sys.argv:
+    if reuse_queue:
         # command line flag to enable queue
         queued_urls = queue.pull(limit=max_queue)
     else:
@@ -731,17 +772,17 @@ def threaded_get(url=None, urls=None, num_threads=10, dl=None, cb=None, depth=No
     else:
         # remove any queued URL's so can crawl again
         queue.clear()
+        urls = urls or []
         if url:
-            queue.push([url])
-        if urls:
-            queue.push(urls)
+            urls.append(url)
+        queue.push([(url, DEFAULT_PRIORITY) for url in urls])
         # put urls into thread safe queue
         seed_urls = collections.deque(queue.pull(limit=max_queue))
         common.logger.debug('Start new crawl')
 
     # initiate the state file with the number of URL's already in the queue
     state = State()
-    state.update(queue_size=len(queue) or len(seed_urls))
+    state.update(queue_size=len(queue))
 
     # start the download threads
     threads = [DownloadThread() for i in range(num_threads)]
@@ -800,8 +841,7 @@ class State:
         self.num_downloads += num_downloads
         self.num_errors += num_errors
         self.num_caches += num_caches
-        if queue_size:
-            self.queue_size = queue_size
+        self.queue_size = queue_size
         self.data['num_downloads'] = self.num_downloads
         self.data['num_errors'] = self.num_errors
         self.data['num_caches'] = self.num_caches
@@ -819,18 +859,21 @@ class State:
         self.data['duration_secs'] = int(self.last_time - self.start_time)
         self.flush = False
         text = json.dumps(self.data)
-        tmp_file = self.output_file + '.tmp'
+        tmp_file = '%s.%d' % (self.output_file, os.getpid())
         fp = open(tmp_file, 'wb')
         fp.write(text)
         # ensure all content is written to disk
         fp.flush()
         fp.close()
-        if os.name == 'nt': 
-            # on wineows can not rename if file exists
-            if os.path.exists(self.output_file):
-                os.remove(self.output_file)
-        # atomic copy to new location so state file is never partially written
-        os.rename(tmp_file, self.output_file)
+        try:
+            if os.name == 'nt': 
+                # on wineows can not rename if file exists
+                if os.path.exists(self.output_file):
+                    os.remove(self.output_file)
+            # atomic copy to new location so state file is never partially written
+            os.rename(tmp_file, self.output_file)
+        except OSError:
+            pass
 
 
 
@@ -867,6 +910,9 @@ class CrawlerCallback:
         self.robots = robots
         self.crawl_existing = crawl_existing
 
+    def __call__(self, D, url, html):
+       # add scraping code here ...
+       return self.crawl(D, url, html)                                                                                                          
 
     def crawl(self, D, url, html): 
         """Crawl website html and return list of URLs crawled
